const fs = require('fs');
const logger = require('../adapters/winston.logger');
const crypto = require("crypto");

const NO_IMAGE_PATH = './images/no-image.dat';
const BASE64_IMAGE_NOTATION_REPLACER_REGEX = /^data:image\/(png|jpeg|jpg);base64,/;
const IMAGE_TYPE_REGEX = /^data:(image\/(png|jpeg|jpg))/;

const FILE_FORMAT_EXTENSIONS = {
    'JPEG': '.jpg'
}

const FILE_FORMAT_MIMES = {
    'JPEG': 'image/jpeg'
}

const FILE_MIME_TYPES_EXTENSIONS = {
    'image/jpeg': '.jpg'
}

const RANDOM_STRING_LENGTH = 16;
const RANDOM_STRING_OUTPUT = 'hex';

/**
 * Exported utilities methods.
 */
module.exports = {

    /**
     * Generate a random string.
     * @returns {string}
     */
    generateRandomString(){
        return crypto.randomBytes(RANDOM_STRING_LENGTH).toString(RANDOM_STRING_OUTPUT);
    },

    /**
     *
     * @param mimeType {String} - Mime type.
     * @returns {String}
     */
    getImageExtensionFromMimeType: function(mimeType){
        return FILE_MIME_TYPES_EXTENSIONS[mimeType];
    },

    /**
     * Calculates and return the file extension based on the file format originated from the camera.
     * @param fileFormat {String}
     * @returns {string}
     */
    getImageExtension: function(fileFormat){

        if( fileFormat in FILE_FORMAT_EXTENSIONS ){
            return FILE_FORMAT_EXTENSIONS[fileFormat];
        }
        logger.warn("File format "+fileFormat+" not found, return default..");
        return '.jpg';
    },

    /**
     * Calculates and return the file extension based on the file format originated from the camera.
     * @param fileFormat {String}
     * @returns {string}
     */
    getImageMime: function(fileFormat){

        if( fileFormat in FILE_FORMAT_MIMES ){
            return FILE_FORMAT_MIMES[fileFormat];
        }
        logger.warn("File format "+fileFormat+" not found, return default..");
        return 'image/jpeg';
    },

    /**
     * Get an image as base64 and return the type and binary of this image.
     * @param image - The base64 of the image
     * @returns {{binary: Buffer, type: string}}
     */
    getImage: function (image){
        let base64Data = image.replace(BASE64_IMAGE_NOTATION_REPLACER_REGEX, '');
        let matches = IMAGE_TYPE_REGEX.exec(image);
        let imageType = 'image/*';
        if ( matches != null ){
            imageType = matches[1];
        }
        return {
            'type':  imageType,
            'binary': Buffer.from(base64Data, 'base64')
        }
    },

    /**
     * Return the no-image
     */
    noImage: function (){
        return new Promise((resolve, reject) => {
            fs.readFile(NO_IMAGE_PATH, {}, (err, dat) => {
                if( err ) reject(new Error("No image found..."));
                resolve(this.getImage(dat.toString()));
            })
        });
    }

}
